;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;    this file is part of CavityStuffer - a molecular polymer design tool
;;;                         -------------
;;; copyright (c) 1993 by Institute for Molecular Manufacturing, Palo Alto, CA
;;; copyright (c) 1994 and beyond: M. Krummenacker (KR) kr@shell.portal.com
;;;
;;; -This program is free software; you can redistribute it and/or modify
;;;   it under the terms of the GNU General Public License as published by
;;;   the Free Software Foundation.
;;; -This program is distributed in the hope that it will be useful, but WITHOUT
;;;   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;;;   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.
;;; -You should have received a copy of the GNU General Public License along with
;;;   this program (the file "COPYING-2.0"); if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#-(or GCL ECL)
(cl:defpackage "CAVSTUF"
  (:nicknames "CVST")
  (:use "COMMON-LISP")
  (:export )
  )
;;; in MCL, see below for (shadow 'read-line)

#|
;;; probably the (in-package ...) below takes care of this already, as it
;;; ought to implicitly (make-package ...) in the old pre-CLtL2 LISPs
#+(or GCL ECL)
(make-package "CAVSTUF"
	      :nicknames '("CVST")
	      :use '("LISP")
	      )
|#

;;; for all the old-fashioned prehistoric LISPs that do not have the new
;;; packages as required by CLtL2
(unless (find-package "CL")
	(make-package "CL"
		      :use '("LISP")
		      )
	)

(in-package "CAVSTUF")

#-(or GCL ECL)  ;; these crappy old LISPs still do not seem to have this :-(
(setf *default-pathname-defaults*
      (make-pathname :directory (pathname-directory *LOAD-PATHNAME*)
		     ))

#|crappy stuff
(setf *default-pathname-defaults*
      (make-pathname :directory
                     #+(or GCL ECL)
		     '(:absolute "home" "kr" "cavstuf"
				 "cavstuf-0.2.2")
		     #-(or GCL ECL)
		     (directory-namestring
                      cl:*load-pathname*
#|
                      #-MCL cl:*load-pathname*
                      #+MCL *loading-file-source-file*
|#
                      )))
|#

#|;;; should we use something like this ?   kr 95-9-11
(def-logical-pathname "modules-root" 
  (directory-namestring *loading-file-source-file*))
;;; or this, from zebu ??
(let ((*default-pathname-defaults*
       (make-pathname :directory (pathname-directory *LOAD-PATHNAME*)
		      :type "lisp"))
|#

;;; used in the (read-line ...) replacement below
#+MCL
(defun string-insert-extend (new-element
                             vector
                             &optional (position (length vector)))
  "new-element
   vector              ; has to be :adjustable, but no :fillpointer!!
   &optional position  ; default if not supplied is the length of the vector
Takes the new-element and inserts it into the vector at the specified
position (counting from zero, as usual). Adjusts the length of the vector
beforehand if the position lies outside the previous length."
  (if (>= position (length vector))
    (setf vector (adjust-array vector
                               (+ 1 position)
                             ;;  :initial-element nil
                               )))
  (setf (elt vector position) new-element))

;; insert setting *warn-if-redefine-kernel* to nil for the following in MCL:
;; (kr 95-7-28) (but see below for something much better!)
;;; use (shadow ...) to avoid the conflict. kr 95-10-10
#+MCL
(shadow 'read-line)

;;; replace (read-line ...) with a look-alike that allows slurping up files
;;;  with Mac CR as well as UN!X LF
;;;;not implemented yet: possibly a problem: treat eof-error-p properly!!!!!
#+MCL
(defun read-line (&optional input-stream
                            eof-error-p
                            eof-value
                            recursive-p)
  "input-stream
   eof-error-p
   eof-value
   recursive-p
An emulation which tolerates new-line characters from UN!X, Mac, and even PC clones.
The resulting string does not contain a new-line character.
Has an enhancement: returns as the third value the type of the new-line read."
  (let ((result-string (make-array 0
                                   :element-type 'character
                                   :adjustable t
                                 ;;  :initial-element nil
                                   ))
        (eof-flag nil)
        (line-type-flag nil))
    (do ((current-character)
         (line-end-flag nil))
        (line-end-flag                    ; end-test: return the string
         (values result-string
                 eof-flag
                 line-type-flag))         ; return some interesting info
      (setf current-character
            (read-char input-stream nil 'eof recursive-p))
      (case current-character
        ('eof
         (progn
           (setf eof-flag t)              ; abnormal line termination as a result
           (setf line-end-flag t)         ; let the end-test terminate this line
           (setf line-type-flag 'eof)))
  ;;      (#\Newline             ; generic Common LISP (actually this is erronous !)
  ;;       (setf end-flag t))    ; let the end-test terminate this line
        (#\Linefeed                       ; UN!X
         (progn
           (setf line-end-flag t)         ; let the end-test terminate this line
           (setf line-type-flag 'un!x)))
        (#\Return                         ; Mac, and possibly even a screwy PC clone
         (let ((one-character-buffer
                (read-char input-stream eof-error-p eof-value recursive-p)))
           (if (equal one-character-buffer #\Linefeed)
             ;; now we are truly faced with one of these PC clones
             (progn
               (setf line-end-flag t)     ; let the end-test terminate this line
               (setf line-type-flag 'ibm))
             ;; it's just a Mac: restore this fetched look-ahead again
             (progn
               (unread-char one-character-buffer input-stream)
               (setf line-end-flag t)     ; let the end-test terminate this line
               (setf line-type-flag 'mac)))))
        (otherwise (string-insert-extend current-character result-string)))
      )))
#| just a few tests
(setf file (open "MacPartition:cubane.gnu.pdb"))
(setf file (open "MacPartition:cubane.mac.pdb"))
(setf file (open "MacPartition:cubane.ibm.pdb"))
(setf file (open "MacPartition:cubane.trunc.pdb"))
(read-line file)
(read-char file nil 'eof)
|#

;;; set some directory paths

(defparameter PDB-moiety-directory 
  ;; sorted in order of increasingly brain-dead lack of features
  #-(or GCL ECL CLISP)
  '(:relative :up  "PDB_files" "PDBmoieties0.1")
  #+CLISP
  '(:relative ".." "PDB_files" "PDBmoieties0.1")
  #+(or GCL ECL)
  '(          ".." "PDB_files" "PDBmoieties0.1")
  "This file-directory is used by the PDB-parsing routines 
for reading moieties while constructing the moiety library.")

(defparameter PDB-CavStuf-directory 
  ;; sorted in order of increasingly brain-dead lack of features
  #-(or GCL ECL CLISP)
  '(:relative :up  "PDB_files" "PDBCavStuf")
  #+CLISP
  '(:relative ".." "PDB_files" "PDBCavStuf")
  #+(or GCL ECL)
  '(          ".." "PDB_files" "PDBCavStuf")
  "This file-directory is used by the PDB-parsing routines 
for reading moieties while constructing the moiety library.")

(defparameter Lib-directory
  ;; sorted in order of increasingly brain-dead lack of features
  #-(or GCL ECL CLISP)
  '(:relative :up  "CavStuf-Libraries")
  #+CLISP
  '(:relative ".." "CavStuf-Libraries")
  #+(or GCL ECL)
  '(          ".." "CavStuf-Libraries")
  "This file-directory is used by the Moiety-type-Library
managing routines.")

(defparameter version-string "CavStuf-0.2.2"
  "This string defines the version number of CavStuf 
and is referenced during writing to files, for example.")

;;; use the following flag to suppress writing white-space 
;;;  to fill precisely 80 columns in PDB files.
;;;  default is turned on. turn off with nil.
(setf *write-space-saving-pdb-files* t)

;;; a general language-addition :
(defmacro dosequence ((var sequenceform &optional resultform)
                       &body body)
"Generalizes dolist to any kind of sequence."
  (let ((position (gensym "DOSEQUENCE")))
    `(let ((seq-len (length ,sequenceform)))
       (do ((,position 0 (1+ ,position)))
           ((>= ,position seq-len)
            (let ((,var nil))
              ,var                 ;use it trivially.
              ,resultform))
         (let ((,var (elt ,sequenceform ,position)))
           ,@body)))))

(defmacro doseq-with-index ((var
                               index
                               sequenceform
                               &optional resultform)
                              &body body)
"Generalizes dolist to any kind of sequence.
And it caries along with it the counting index."
  `(do ((,index 0 (1+ ,index)))
       ((>= ,index (length ,sequenceform))
        (let ((,var nil))
          ,var                 ;use it trivially.
          ,resultform))
     (let ((,var (elt ,sequenceform ,index)))
       ,@body)))

;;; load in all the relevant files
(let ((cavstuf-files (list
		      "structure-defs"
		      "matrix-vector"
		      "norm-map"
		      "file-library"
		      "pdb-in-out"
		      "create-moiety-type"
		      "create-link-type"
		      "clash"
		      "tree-anchorage"
		      "tree-growth"
		      "tree-dump"
		      "growth-driver"
		      "biased-random"
		      "atm-properties"
		      "tree-shorthand"
                      ) ; list
		     ))
  (dosequence (file cavstuf-files)
     (load (concatenate 'string file ".lisp"))
     )
  ) ; let

(print (format nil "~A loaded" version-string))
