;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;    this file is part of CavityStuffer - a molecular polymer design tool
;;;                         -------------
;;; copyright (c) 1993 by Institute for Molecular Manufacturing, Palo Alto, CA
;;; copyright (c) 1994 and beyond: M. Krummenacker (KR) kr@shell.portal.com
;;;
;;; -This program is free software; you can redistribute it and/or modify
;;;   it under the terms of the GNU General Public License as published by
;;;   the Free Software Foundation.
;;; -This program is distributed in the hope that it will be useful, but WITHOUT
;;;   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;;;   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.
;;; -You should have received a copy of the GNU General Public License along with
;;;   this program (the file "COPYING-2.0"); if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package "CAVSTUF")   ;; added this 95-5-6, modified to string 95-10-10

;;; General comment:
;;; This is just an old hack, by using CommonLISP very inelegantly as a 
;;; lowly scripting language, with a lot of duplicative code. 
;;; Ideally, more useful entry tools should be written, which are also 
;;; more intelligent in terms of generating rotamers for different conformations, 
;;; without having to explicitly enumerate all of them in this kludged script.

(progn

  ;; used for speed purposes
  (internalize-hash-table 'pdb-Chem3D+)
 ;; (setf PDB-directory "MacPartition:PDB Reservoire 0.1")
  (init-mlib 'CyHxamide0.1)

  ;; The following is a _hack_! But it is necessary to discard the pointers in the
  ;;   atm-structures to avoid forming poly-cyclic data structures which
  ;;   provide certain awkward problems in printing out to a file...
  ;;;(actually: updated comment: just knowing how to properly use the right 
  ;;; print flags would take care of this problem. I just did not know about 
  ;;; them in those days :-)
  ;; The proper, full-size definition has to be restored at the end of this
  ;;   "script". It should be identical to the one in the file "structure-defs".
  (defstruct atm
    (xyz #(0.0 0.0 0.0) :type (vector 'real))
    type)               ; type of the atom (an integer)

  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "mirrorcubane.pdb" pdb-Chem3D+))
      ;    (read&parse-all-lines-of-a-file "cubane.pdb" pdb-Chem3D+))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "Cubane-frame"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (4) (12))
                         ;      (8) (12))
                              (1
                               (6) (14))
                              (2
                               (2) (10))
                              (3
                               (8) (16))
                          ;     (4) (16))
                              )
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "amide.pdb" pdb-Chem3D+))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "amide"
           :raw-format      raw-format
           :coordinate-atms '(1 2 4)
           :sites           '((0
                               (1) ())
                              (1
                               (2) ()))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "CyHx.pdb" pdb-Chem3D+))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "CyHxbetaA"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (1) (7))
                              (1
                               (2) (9)))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "CyHxbetaB"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (1) (7))
                              (1
                               (6) (17)))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "CyHxgammaA"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (1) (7))
                              (1
                               (3) (11)))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "CyHxgammaB"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (1) (7))
                              (1
                               (5) (15)))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    (setf sample-moiety-type
          (create-moiety-type
           :name            "CyHxdelta"
           :raw-format      raw-format
           :coordinate-atms '(1 2 3)
           :sites           '((0
                               (1) (7))
                              (1
                               (4) (13)))
           ))
    (add-to-mlib sample-moiety-type)
    (print (moiety-type-name sample-moiety-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-cub4->amide1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       0
        ;   :parent-bonds      '((5 (0)))
           :parent-bonds      '((7 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(1 2 3)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(7 11 0 2 6)
           :lk-name           "cub0->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       0
        ;   :parent-bonds      '((5 (0)))
           :parent-bonds      '((7 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(3 8 5)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(7 11 0 2 6)
           :lk-name           "cub0->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       0
        ;   :parent-bonds      '((5 (0)))
           :parent-bonds      '((7 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(5 6 1)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(7 11 0 2 6)
           :lk-name           "cub0->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-cub4->amide1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       1
         ;  :parent-bonds      '((3 (0)))
         ;  :parent-bonds      '((1 (0)))
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(5 8 7)
       ;    :parent-coord-atms '(3 4 1)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(5 13 0 4 6)
           :lk-name           "cub1->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       1
         ;  :parent-bonds      '((3 (0)))
         ;  :parent-bonds      '((1 (0)))
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
     ;;      :parent-coord-atms '(7 2 1)
           :parent-coord-atms '(1 6 7)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(5 13 0 4 6)
           :lk-name           "cub1->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       1
         ;  :parent-bonds      '((3 (0)))
         ;  :parent-bonds      '((1 (0)))
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
    ;;       :parent-coord-atms '(1 4 5)
    ;;       :parent-coord-atms '(7 8 3)
           :parent-coord-atms '(3 2 7)
      ;;     :parent-coord-atms '(5 6 1)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(5 13 0 4 6)
           :lk-name           "cub1->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-cub4->amide1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       2
          ; :parent-bonds      '((5 (0)))
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
       ;    :parent-coord-atms '(5 8 7)
           :parent-coord-atms '(3 4 1)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(1 9 0 2 4)
           :lk-name           "cub2->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       2
          ; :parent-bonds      '((5 (0)))
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(5 4 3)
      ;;     :parent-coord-atms '(7 2 1)
     ;;      :parent-coord-atms '(1 6 7)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(1 9 0 2 4)
           :lk-name           "cub2->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       2
          ; :parent-bonds      '((5 (0)))
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(1 4 5)
     ;;      :parent-coord-atms '(7 8 3)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(1 9 0 2 4)
           :lk-name           "cub2->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-cub4->amide1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       3
        ;   :parent-bonds      '((7 (0)))
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 8 3)
    ;;       :parent-coord-atms '(5 6 1)
    ;;       :parent-coord-atms '(3 2 7)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(3 15 2 4 6)
           :lk-name           "cub3->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       3
        ;   :parent-bonds      '((7 (0)))
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 6 5)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(3 15 2 4 6)
           :lk-name           "cub3->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "Cubane-frame"
           :parent-site       3
        ;   :parent-bonds      '((7 (0)))
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 2 1)
      ;;     :parent-coord-atms '(5 4 3)
           :child-coord-atms  '(12 17 20)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);'(3 15 2 4 6)
           :lk-name           "cub3->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-amide->CyHxup1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaA1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaB1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaA1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaB1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
#|
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxdelta"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxdelta1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
|#
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-amide->CyHxup2.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaA2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaB2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaA2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaB2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
#|
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxdelta"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxdelta2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
|#
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-amide->CyHxup3.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaA3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxbetaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxbetaB3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaA"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaA3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxgammaB"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxgammaB3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
#|
    (setf sample-link-type
          (create-link-type
           :parent-name       "amide"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "CyHxdelta"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(7 19 22)
           :child-coord-atms  '(1 2 3)
           :clash-atms        '(0 1 2 3)
           :lk-name           "amide->CyHxdelta3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
|#
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-CyHxup->amide1.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaA"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(6 5 4)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(1 8)
           :lk-name           "CyHxbetaA->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaB"
           :parent-site       1
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(4 3 2)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(3 16)
           :lk-name           "CyHxbetaB->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaA"
           :parent-site       1
           :parent-bonds      '((2 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(3 4 5)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(2 10)
           :lk-name           "CyHxgammaA->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaB"
           :parent-site       1
           :parent-bonds      '((4 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(1 2 3)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(4 14)
           :lk-name           "CyHxgammaB->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxdelta"
           :parent-site       1
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(2 1 6)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(5 12)
           :lk-name           "CyHxdelta->amide1"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-CyHxup->amide2.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaA"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(6 5 4)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(1 8)
           :lk-name           "CyHxbetaA->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaB"
           :parent-site       1
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(4 3 2)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(3 16)
           :lk-name           "CyHxbetaB->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaA"
           :parent-site       1
           :parent-bonds      '((2 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(3 4 5)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(2 10)
           :lk-name           "CyHxgammaA->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaB"
           :parent-site       1
           :parent-bonds      '((4 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(1 2 3)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(4 14)
           :lk-name           "CyHxgammaB->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxdelta"
           :parent-site       1
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(2 1 6)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(5 12)
           :lk-name           "CyHxdelta->amide2"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )
  (progn
    (setf raw-format
          (read&parse-all-lines-of-a-file "lk-CyHxup->amide3.pdb" pdb-Chem3D+))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaA"
           :parent-site       1
           :parent-bonds      '((1 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(6 5 4)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(1 8)
           :lk-name           "CyHxbetaA->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxbetaB"
           :parent-site       1
           :parent-bonds      '((3 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(4 3 2)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(3 16)
           :lk-name           "CyHxbetaB->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaA"
           :parent-site       1
           :parent-bonds      '((2 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(3 4 5)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(2 10)
           :lk-name           "CyHxgammaA->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxgammaB"
           :parent-site       1
           :parent-bonds      '((4 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(1 2 3)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(4 14)
           :lk-name           "CyHxgammaB->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    (setf sample-link-type
          (create-link-type
           :parent-name       "CyHxdelta"
           :parent-site       1
           :parent-bonds      '((5 (0)))
           :child-name        "amide"
           :child-site        0
           :raw-format        raw-format
           :parent-coord-atms '(2 1 6)
           :child-coord-atms  '(15 19 22)
           :clash-atms        '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);'(5 12)
           :lk-name           "CyHxdelta->amide3"
           ))
    (add-link-type-to-mlib sample-link-type)
    (print (link-type-name sample-link-type))
    )

  ;; The print-level has to be set to no restrictions for printing out the mlib
  (let ((previous-print-level *print-level*))
    (setf *print-level* nil)

    (save-mlib)

    ;; Restore the original value of print-level
    (setf *print-level* previous-print-level)
    ;; Terminate the let:
    )

  ;; This brings the _hack_ to completion:
  (defstruct atm
    (xyz #(0.0 0.0 0.0) :type (vector 'real)) ; vector of coordinates,
                                              ;   #(x y z), in pm
    type          ; type (an integer)
    (image nil)   ; pointer to next in a possible chain of images;
                  ;    can store a global atom number when walking
                  ;    the tree to construct a PDB file
    affiliation)  ; pointer to the moiety of which the atm is part of
  
  (values)    ; don't return any spurious result from this ugly script
)
