;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;    this file is part of CavityStuffer - a molecular polymer design tool
;;;                         -------------
;;; copyright (c) 1993 by Institute for Molecular Manufacturing, Palo Alto, CA
;;; copyright (c) 1994 and beyond: M. Krummenacker (KR) kr@shell.portal.com
;;;
;;; -This program is free software; you can redistribute it and/or modify
;;;   it under the terms of the GNU General Public License as published by
;;;   the Free Software Foundation.
;;; -This program is distributed in the hope that it will be useful, but WITHOUT
;;;   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;;;   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.
;;; -You should have received a copy of the GNU General Public License along with
;;;   this program (the file "COPYING-2.0"); if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package "CAVSTUF")


(defun atm-type=H-p (atm)
  (if (eq (atm-type atm) 1) t nil))

(defun atm-type=C-p (atm)
  (if (eq (atm-type atm) 2) t nil))

(defun atm-type=N-p (atm)
  (if (eq (atm-type atm) 3) t nil))

(defun atm-type=O-p (atm)
  (if (eq (atm-type atm) 4) t nil))

(defun atm-type=F-p (atm)
  (if (eq (atm-type atm) 5) t nil))

(defun atm-type=Si-p (atm)
  (if (eq (atm-type atm) 6) t nil))

(defun atm-type=P-p (atm)
  (if (eq (atm-type atm) 7) t nil))

(defun atm-type=S-p (atm)
  (if (eq (atm-type atm) 8) t nil))

(defun atm-type=Cl-p (atm)
  (if (eq (atm-type atm) 9) t nil))

(defun generate-raw-format (parsed-atm-vector parsed-bond-starlet-vector)
  (list parsed-atm-vector parsed-bond-starlet-vector))

(defun read&parse-all-lines-of-a-file (file-name file-format)
  "file-name ;file as a string, including extension
   file-format ;the format translation-table
Reads-in a file that specifies the atom coordinates and connectivity
of a moiety-type. Returns raw-format, which is a list of two vectors,
the first containing parsed-atms, the second parsed-bond-starlets"
  (let ((atm-vector  (make-array 0 :adjustable t
                                   :fill-pointer 0))
        (bond-starlet-vector (make-array 0 :adjustable t
                                           :fill-pointer 0))
        (one-line-string ""))
    (with-open-file (in-filestream 
                     (make-pathname :directory PDB-moiety-directory
                                    :name file-name)
                     :direction :input)
      (do ((line-counter 1 (1+ line-counter))
           (end-of-file-flag nil))
          (end-of-file-flag (error "EOF occurred within line ~D"
                                   line-counter))
        (multiple-value-setq
         (one-line-string
          end-of-file-flag)
         (read-line in-filestream))
;                    nil
;                    (error "EOF occurred in line ~D,\
;         before valid terminating line was encountered" line-counter)))
        (multiple-value-bind (atm bonds control)
                             (parse-line one-line-string
                                         file-format
                                         line-counter)
          (ecase control
                 (atm (vector-push-extend atm atm-vector))
                 (bonds (vector-push-extend bonds bond-starlet-vector))
                 (atms&bonds (vector-push-extend atm atm-vector)
                              (vector-push-extend bonds bond-starlet-vector))
                 (not-recognized nil)
                 (end-reached (return (generate-raw-format
                                          atm-vector
                                          bond-starlet-vector)))))))))

(defun parse-line (one-line-string translation-table line-number)
  "one-line-string   ;on line of a file
   translation-table ;the format translation-table
   line-number       ;number of line in file, for error information
Works in close conjunction with read&parse-all-lines-of-a-file.
Returns three values, the last of which acts as a control channel.
A large part of the parsing instructions are actually contained
in the translation-table, within of which the returned values are generated
whenever the first word of a line is recognized"
  (multiple-value-bind (operator found-flag)
           (gethash (string-right-trim Whitespace-filter
                                       (subseq one-line-string 0 
                                               (if (<
                                                    (length one-line-string)
                                                    6)
                                                 (length one-line-string)
                                                 6)))
                    translation-table)
     (if (eql found-flag nil)
         (values nil nil 'not-recognized)
         (if (equal (first operator) "x")
           (funcall (second operator)                ; executable directly
                  one-line-string translation-table line-number)
           (funcall (eval (second operator))         ; needs to be compiled first
                  one-line-string translation-table line-number)))))

#|
(progn (setf pdb-parse
             (read&parse-all-lines-of-a-file "norbornyl.pdb" pdb-Chem3D+))
       (setf parsed-atm-vector (first pdb-parse))
       (setf bond-vector (second pdb-parse)))
(read&parse-all-lines-of-a-file "trash.pdb" pdb-Chem3D+)
(read&parse-all-lines-of-a-file "norbornyl.pdb" pdb-Chem3D+)
(read&parse-all-lines-of-a-file "trispikepoly.pdb" pdb-Chem3D+)
(parse-line "COMPND  Norbornyl                                         "                         
            pdb-Chem3D+ 1)
(parse-line
"HETATM    1  O                 -1.0556  0.0677  0.1841                "
            pdb-Chem3D+ 3)
(parse-line
"HETATM  228  C       A 228      28.219  27.256  23.069                "
            pdb-Chem3D+ 3)
(parse-line "CONECT    7    1    4   18   19     "                         
            pdb-Chem3D+ 2)
chokes with an error if integers are malformed:
(parse-line "CONECT screwed-up    1    4   18   19     "                         
            pdb-Chem3D+ 3)
(parse-line "CONECT   12    3                                             "                         
            pdb-Chem3D+ 4)
(parse-line
"CONECT    7   11"
            pdb-Chem3D+ 3)
(parse-line "END                         "                         
            pdb-Chem3D+ 5)
(parse-line "END"                         
            pdb-Chem3D+ 5)
|#
;;; Construct an initial minimal translation-table pdb-Chem3D+
(progn (setf pdb-Chem3D+ (make-hash-table :test #'equal))
        (setf (gethash 'initial-state pdb-Chem3D+) '(tbd))
        (setf (gethash "CONECT" pdb-Chem3D+)
              '("s" #'(lambda (one-line-string dummy1 dummy2)
                        (let ((center-atm
                               (parse-integer one-line-string
                                              :start 6 :end 11))
                              (line-length (length one-line-string)))
                          (do ((end-of-line-flag nil)
                               (temp-atm nil)
                               (index 11)
                               (bond-ends-list '()))
                              (end-of-line-flag
                               (if (eql bond-ends-list '())
                                 (values nil
                                         nil
                                         'not-recognized)
                                 (values nil
                                         ;; put together a bond-starlet:
                                         (list center-atm bond-ends-list)
                                         'bonds)))
                            (multiple-value-setq
                             (temp-atm
                              index)                       ;;index gets advanced
                             (parse-integer one-line-string
                                            :start index 
                                            :end (+ index 5)
                                            :junk-allowed T))
                            (if (eql temp-atm nil)         ;;Test if integer
                              (setf end-of-line-flag T)    ;;  was (malformed)
                              (setf bond-ends-list         ;;  or non-existant
                                    (nconc bond-ends-list (list temp-atm))))
                            (if (>= index line-length)
                              (setf end-of-line-flag T)
                              nil))))))
        (setf (gethash "HETATM" pdb-Chem3D+)
              '("s" #'(lambda (one-line-string translation-table line-number)
                        (let* ((atm-number
                                (parse-integer one-line-string
                                               :start 6 :end 11))
                               (atm-label
                                (string-trim Whitespace-filter
                                             (subseq one-line-string 12 16)))
                               (xyz (vector
                                     (* 100 (read-from-string
                                             one-line-string T nil
                                             :start 30 :end 38))
                                     (* 100 (read-from-string
                                             one-line-string T nil
                                             :start 38 :end 46))
                                     (* 100 (read-from-string
                                             one-line-string T nil
                                             :start 46 :end 54)))))
                          (multiple-value-bind (atm-kind found-flag)
                                               (gethash atm-label
                                                        translation-table)
                            (if (eql found-flag nil)
nil
               ;         (error \unknown atom label in pdb-file; in line ~D\
               ;                line-number)
                            (values (list atm-number
                                    (make-atm :xyz  xyz
                                              :type (gethash atm-kind
                                                       translation-table)))
                                  nil
                                  'atm)))))))
        (setf (gethash "END" pdb-Chem3D+)
              '("s" #'(lambda (dummy1 dummy2 dummy3)
                        (values nil
                                nil
                                'end-reached))))
        (setf (gethash "H" pdb-Chem3D+) 'hydrogen)
        (setf (gethash "C" pdb-Chem3D+) 'carbon)
        (setf (gethash "N" pdb-Chem3D+) 'nitrogen)
        (setf (gethash "O" pdb-Chem3D+) 'oxygen)
        (setf (gethash "F" pdb-Chem3D+) 'fluorine)
        (setf (gethash "Si" pdb-Chem3D+) 'silicon)
        (setf (gethash "P" pdb-Chem3D+) 'phosphorous)
        (setf (gethash "S" pdb-Chem3D+) 'sulfur)
        (setf (gethash "Cl" pdb-Chem3D+) 'chlorine)
        (setf (gethash "Br" pdb-Chem3D+) 'bromine)
        (setf (gethash 'hydrogen pdb-Chem3D+) 1)
        (setf (gethash 'carbon pdb-Chem3D+) 2)
        (setf (gethash 'nitrogen pdb-Chem3D+) 3)
        (setf (gethash 'oxygen pdb-Chem3D+) 4)
        (setf (gethash 'fluorine pdb-Chem3D+) 5)
        (setf (gethash 'silicon pdb-Chem3D+) 6)
        (setf (gethash 'phosphorous pdb-Chem3D+) 7)
        (setf (gethash 'sulfur pdb-Chem3D+) 8)
        (setf (gethash 'chlorine pdb-Chem3D+) 9)
        (setf (gethash 'bromine pdb-Chem3D+) 10)
        (values (dump-hash-table pdb-Chem3D+)
                pdb-Chem3D+))

(defun internalize-hash-table (table-symbol)
  (let ((old-table (symbol-value table-symbol)))
    (setf (symbol-value table-symbol) (make-hash-table :test #'equal))
    (maphash #'(lambda (key val)
                 (if (and (eql (type-of val) 'cons)
                          (equal (first val) "s"))
                   (setf (gethash key (symbol-value table-symbol))
                         (list "x" (eval (second val))))
                   (setf (gethash key (symbol-value table-symbol))
                         val)))
             old-table)
    (symbol-value table-symbol)))


(defun generate-parsed-atm-vector (atm-array &optional (offset 0))
  "atm-array
   &optional offset  ;defaults to zero
Generate a vector of parsed-atms which correspond in their sequence
to the atm-array, the numbering of the parsed-atms starting at offset"
  (let ((parsed-atm-vector  (make-array 0 :adjustable t
                                        :fill-pointer 0))
        (this-many (length atm-array)))
    (dotimes (index this-many)
      (vector-push-extend (list (+ index offset) (elt atm-array index))
                          parsed-atm-vector))
    (values parsed-atm-vector (+ this-many offset))))

(defun generate-parsed-bond-starlet-vector (bond-array
                                                 &optional (offset 0))
  "bond-array
   &optional offset  ;defaults to zero
Generate a vector of parsed-bond-starlets which correspond in their
sequence to the bond-array, the numbering of the center-atms starting at offset
and the end-atms corrected with offset"
  (let ((parsed-bond-starlet-vector  (make-array 0 :adjustable t
                                                   :fill-pointer 0)))
    (dotimes (index (length bond-array) parsed-bond-starlet-vector)
      (vector-push-extend (list (+ index offset)
                                (map 'list
                                     #'(lambda (end-atm)
                                         (+ end-atm offset))
                                     (elt bond-array index)))
                          parsed-bond-starlet-vector))))

(defun write-pdb-file (raw-format file-name)
  "raw-format
   file-name ;file as a string, including extension
Writes-out a file in PDB-format that specifies the atom coordinates and
connectivity as indicated in raw-format, which is a list of two vectors,
the first containing parsed-atms, the second parsed-bond-starlets"
  (with-open-file (out-filestream 
                   (make-pathname :directory PDB-CavStuf-directory
                                  :name file-name)
                   :direction :output
                   :if-does-not-exist :create
                   :if-exists :supersede)
    (write-line (generate-COMPND-pdb-line file-name)
                out-filestream)
    (write-line (generate-SOURCE-pdb-line)
                out-filestream)
    (write-line (generate-AUTHOR-pdb-line)
                out-filestream)
    (write-line (generate-EXPDTA-pdb-line)
                out-filestream)
    (dosequence (parsed-atm (first raw-format))
                (unless (eql parsed-atm '())
                  (write-line (generate-HETATM-pdb-line parsed-atm)
                              out-filestream)))
    (dosequence (bond-starlet (second raw-format))
                (write-line (generate-CONECT-pdb-line bond-starlet)
                            out-filestream))
    (write-line (generate-END-pdb-line)
                out-filestream)))

#-MCL
(defun generate-HETATM-pdb-line (parsed-atm)
  "parsed-atm
Generates a string corresponding to a HETATM-line in the official
80 column PDB-format. The atm-numbers get incremented by one, so that
number 0 does not appear in the file"
  (let* ((parsed-atm-number (get-parsed-atm-number parsed-atm))
         (type (atm-type (get-parsed-atm parsed-atm)))
         (xyz (atm-xyz (get-parsed-atm parsed-atm)))
         (format-precision (if (some #'(lambda (float-number)
                                        (> float-number 99999.999))
                                    xyz)
                             2
                             3)))
    (concatenate 'string "HETATM"                           ;columns 1-6
                 (format nil "~5D" (+ 1 parsed-atm-number)) ;columns 7-11
                 " "                                        ;column  12
                 (cond ((eql  1 type) " H")
                       ((eql  2 type) " C")
                       ((eql  3 type) " N")
                       ((eql  4 type) " O")
                       ((eql  5 type) " F")
                       ((eql  6 type) "Si")
                       ((eql  7 type) " P")
                       ((eql  8 type) " S")
                       ((eql  9 type) "Cl")
                       ((eql 10 type) "Br")
                       (t             "**"))                ;columns 13-14
                 " "                                        ;column  15
                 " "                                        ;column  16
                 "              "                           ;columns 17-30
                 (format nil "~8,v,-2,'*F"
                             format-precision
                             (elt xyz 0))                   ;columns 31-38
                 (format nil "~8,v,-2,'*F"
                             format-precision
                             (elt xyz 1))                   ;columns 39-46
                 (format nil "~8,v,-2,'*F"
                             format-precision
                             (elt xyz 2))                   ;columns 47-54
                 "                          ")))            ;columns 55-80

;;; The following hack is necessary to camouflage a MCL bug (in MCL-2.0p2) :
;;;  The bug seems to be: (format nil "~8,v,-2,'*F" 3 0.00000001)
;;;  generates four zeros after the point, instead of three.
#+MCL
(defun generate-HETATM-pdb-line (parsed-atm)
  "parsed-atm
Generates a string corresponding to a HETATM-line in the official
80 column PDB-format. The atm-numbers get incremented by one, so that
number 0 does not appear in the file"
  (let* ((parsed-atm-number (get-parsed-atm-number parsed-atm))
         (type (atm-type (get-parsed-atm parsed-atm)))
         (xyz (atm-xyz (get-parsed-atm parsed-atm)))
         (format-precision (if (some #'(lambda (float-number)
                                        (> float-number 99999.999))
                                    xyz)
                             2
                             3)))
    (concatenate 'string "HETATM"                           ;columns 1-6
                 (format nil "~5D" (+ 1 parsed-atm-number)) ;columns 7-11
                 " "                                        ;column  12
                 (cond ((eql  1 type) " H")
                       ((eql  2 type) " C")
                       ((eql  3 type) " N")
                       ((eql  4 type) " O")
                       ((eql  5 type) " F")
                       ((eql  6 type) "Si")
                       ((eql  7 type) " P")
                       ((eql  8 type) " S")
                       ((eql  9 type) "Cl")
                       ((eql 10 type) "Br")
                       (t             "**"))                ;columns 13-14
                 " "                                        ;column  15
                 " "                                        ;column  16
                 "              "                           ;columns 17-30
                 (let ((x (elt xyz 0)))
                   (if (< x 5E-2)
                     (subseq
                      (format nil "~8,v,-2,'*F"
                              format-precision
                              x)
                      0
                      8)
                     (format nil "~8,v,-2,'*F"
                             format-precision
                             x)))                           ;columns 31-38
                 (let ((y (elt xyz 1)))
                   (if (< y 5E-2)
                     (subseq
                      (format nil "~8,v,-2,'*F"
                              format-precision
                              y)
                      0
                      8)
                     (format nil "~8,v,-2,'*F"
                             format-precision
                             y)))                           ;columns 39-46
                 (let ((z (elt xyz 2)))
                   (if (< z 5E-2)
                     (subseq
                      (format nil "~8,v,-2,'*F"
                              format-precision
                              z)
                      0
                      8)
                     (format nil "~8,v,-2,'*F"
                             format-precision
                             z)))                           ;columns 47-54
                 "                          ")))            ;columns 55-80

(defun generate-CONECT-pdb-line (bond-starlet)
  "bond-starlet   ;list: first is center-atm, second the bond-ends-list
Generates a string corresponding to a CONECT-line in the official
80 column PDB-format. The atm-numbers get incremented by one, so that
number 0 does not appear in the file"
  (let* ((trailer-string (make-array 0 :element-type 'character
                                       :adjustable t
                                       :fill-pointer 0))
         (center-atm      (first bond-starlet))
         (target-atm-list (second bond-starlet))
         (targets (length target-atm-list))
         (intermediate-garbage ""))
    (concatenate 'string "CONECT"                           ;columns 1-6
                 (format nil "~5D" (+ 1 center-atm))        ;columns 7-11
                 (dotimes (index targets intermediate-garbage);the next
                   (setf intermediate-garbage                 ;  5 columns
                         (concatenate 'string intermediate-garbage
                                      (format nil "~5D"
                                              (+ 1 (elt target-atm-list
                                                        index))))))
                 (dotimes (index (- 69 (* 5 targets)) trailer-string)
                   (vector-push-extend #\Space
                                       trailer-string)))))  ;fill-in the
                                                            ;  remaining
                                                            ;  columns

(defun generate-COMPND-pdb-line (compound-name)
  "compound-name  ;string"
  (let* ((trailer-string (make-array 0 :element-type 'character
                                       :adjustable t
                                       :fill-pointer 0))
         (suffix-dot-position (if (< (- (length compound-name) 4)
                                     0)
                                0
                                (- (length compound-name) 4)))
         (truncd-name (if (string-equal ".pdb"
                                        (subseq compound-name 
                                                suffix-dot-position))
                        ;; truncate off the .pdb suffix
                        (subseq compound-name 0 suffix-dot-position)
                        compound-name)))
    (concatenate 'string "COMPND"                         ;columns 1-6
                 "    "                                   ;columns 7-10
                 truncd-name                              ;columns 11- ?
                 (dotimes (index
                           (- 70 (length truncd-name))    ;fill-in the
                           trailer-string)                ;  remaining
                   (vector-push-extend #\Space            ;  columns
                                       trailer-string)))))

(defun generate-SOURCE-pdb-line ()
  (concatenate 'string "SOURCE"                           ;columns 1-6
               "    "                                     ;columns 7-10
               "Platonic Heaven                         " ;columns 11-50
               "                              "           ;columns 51-80
               ))

(defun generate-AUTHOR-pdb-line ()
  (let ((v-len (length version-string)))
    (concatenate 'string "AUTHOR"                         ;columns 1-6
                 "    "                                   ;columns 7-10
                 version-string " me fecit"               ;columns 11-
                 (make-string (- 31 v-len)
                              :initial-element #\Space)   ;          -50
                 "                              "         ;columns 51-80
                 )))

(defun generate-EXPDTA-pdb-line ()
  (concatenate 'string "EXPDTA"                           ;columns 1-6
               "    "                                     ;columns 7-10
               "THEORETICAL MODEL                       " ;columns 11-50
               "                              "           ;columns 51-80
               ))

(defun generate-END-pdb-line ()
  (concatenate 'string "END   "                           ;columns 1-6
               "                                  "       ;columns 7-40
               "                                        " ;columns 41-80
               ))

#|
(setf parsed-atm-vector
      (generate-parsed-atm-vector (moiety-type-atm-array sample-moiety-type) 2))
(setf parsed-bond-starlet-vector
      (generate-parsed-bond-starlet-vector
       (moiety-type-bond-array sample-moiety-type) 2))
(generate-raw-format parsed-atm-vector parsed-bond-starlet-vector)
|#


